using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace CommandFramework
{
    public class Command : ICommand
    {
        #region ICommand Members

        public event EventHandler Activated;

        public void Register(object control)
        {
            EventInfo ei = control.GetType().GetEvent("Click");
            if (ei == null)
                return;

            if (!this.controls.Contains(control))
                this.controls.Add(control);

            ei.AddEventHandler(control, new EventHandler(control_Click));
        }

        public void Activate()
        {
            if (this.enabled)
                if (this.Activated != null)
                    this.Activated(this, null);
        }

        public bool Enabled
        {
            get
            {
                return this.enabled;
            }
            set
            {
                this.enabled = value;

                foreach (object control in this.controls)
                {
                    PropertyInfo pi = control.GetType().GetProperty("Enabled");
                    if (pi != null)
                        pi.SetValue(control, value, null);
                }
            }
        }

        #endregion

        #region private

        void control_Click(object sender, EventArgs e)
        {
            this.Activate();
        }

        private List<object> controls = new List<object>();
        private bool enabled = true;

        #endregion
    }
}
