using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;

namespace CommandsOld
{
    public class CommandManager<T> : ICommandManager<T>
    {
        #region ICommandManager<T> Members

        public void Enable(T command)
        {
            this.SetEnabled(command, true);
        }

        public void Disable(T command)
        {
            this.SetEnabled(command, false);
        }

        public void Register(T command, object control)
        {
            EventInfo ei = control.GetType().GetEvent("Click");
            if (ei != null)
            {
                this.controlToCommandLookup.Add(control, command);

                if (!this.commandToControlsLookup.ContainsKey(command))
                    this.commandToControlsLookup.Add(command, new List<object>());

                this.commandToControlsLookup[command].Add(control);
                this.commandToEnabledLookup[command] = true;

                ei.AddEventHandler(control, new EventHandler(control_Click));
            }
        }

        public void Register(T command, CommandHandler<T> callback)
        {
            if (!this.commandToHandlersLookup.ContainsKey(command))
                this.commandToHandlersLookup.Add(command, new List<CommandHandler<T>>());

            this.commandToHandlersLookup[command].Add(callback);
            this.commandToEnabledLookup[command] = true;
        }

        public void Activate(T command)
        {
            if (!this.commandToHandlersLookup.ContainsKey(command))
                return;
            if (!this.commandToEnabledLookup[command])
                return;

            foreach (CommandHandler<T> callback in this.commandToHandlersLookup[command])
                callback.Invoke();
        }
        
        void control_Click(object sender, EventArgs e)
        {
            if (!this.controlToCommandLookup.ContainsKey(sender))
                return;

            T command = this.controlToCommandLookup[sender];
            this.Activate(command);
        }

        void SetEnabled(T command, bool value)
        {
            if (!this.commandToControlsLookup.ContainsKey(command))
                return;

            this.commandToEnabledLookup[command] = value;

            foreach (object c in this.commandToControlsLookup[command])
            {
                PropertyInfo pi = c.GetType().GetProperty("Enabled");
                if (pi != null)
                    pi.SetValue(c, value, null);
            }
        }

        #endregion

        private Dictionary<T, IList<CommandHandler<T>>> commandToHandlersLookup = new Dictionary<T, IList<CommandHandler<T>>>();
        private Dictionary<object, T> controlToCommandLookup = new Dictionary<object, T>();
        private Dictionary<T, IList<object>> commandToControlsLookup = new Dictionary<T, IList<object>>();
        private Dictionary<T, bool> commandToEnabledLookup = new Dictionary<T, bool>();
    }
}
